@@ -11,18 +11,17 @@
 #include "nsNetUtil.h"
 #include <windows.h>
 #include <shellapi.h>
-#include "nsAutoPtr.h"
 #include "nsIMutableArray.h"
 #include "nsTArray.h"
 #include "shlobj.h"
 #include "windows.h"
 #include "nsIWindowsRegKey.h"
-#include "nsIProcess.h"
 #include "nsUnicharUtils.h"
 #include "nsITextToSubURI.h"
 #include "nsVariant.h"
-#include "mozilla/AssembleCmdLine.h"
+#include "mozilla/CmdLineAndEnvUtils.h"
 #include "mozilla/ShellHeaderOnlyUtils.h"
+#include "mozilla/StaticPrefs_browser.h"
 #include "mozilla/UrlmonHeaderOnlyUtils.h"
 #include "mozilla/UniquePtrExtensions.h"
 
@@ -41,10 +40,12 @@ nsresult nsMIMEInfoWin::LaunchDefaultWithFile(nsIFile* aFile) {
   return aFile->Launch();
 }
 
-nsresult nsMIMEInfoWin::ShellExecuteWithIFile(nsIFile* aExecutable,
-                                              const nsString& aArgs) {
+nsresult nsMIMEInfoWin::ShellExecuteWithIFile(nsIFile* aExecutable, int aArgc,
+                                              const wchar_t** aArgv) {
   nsresult rv;
 
+  NS_ASSERTION(aArgc >= 1, "aArgc must be at least 1");
+
   nsAutoString execPath;
   rv = aExecutable->GetTarget(execPath);
   if (NS_FAILED(rv) || execPath.IsEmpty()) {
@@ -54,7 +55,7 @@ nsresult nsMIMEInfoWin::ShellExecuteWithIFile(nsIFile* aExecutable,
     return rv;
   }
 
-  auto assembledArgs = mozilla::assembleSingleArgument(aArgs);
+  auto assembledArgs = mozilla::MakeCommandLine(aArgc, aArgv);
   if (!assembledArgs) {
     return NS_ERROR_FILE_EXECUTION_FAILED;
   }
@@ -75,9 +76,10 @@ nsresult nsMIMEInfoWin::ShellExecuteWithIFile(nsIFile* aExecutable,
   mozilla::LauncherVoidResult shellExecuteOk = mozilla::ShellExecuteByExplorer(
       execPathBStr, assembledArgs.get(), verbDefault, workingDir, showCmd);
   if (shellExecuteOk.isErr()) {
-    // No need to pass assembledArgs to LaunchWithIProcess.  aArgs will be
+    // No need to pass assembledArgs to LaunchWithIProcess.  aArgv will be
     // processed in nsProcess::RunProcess.
-    return LaunchWithIProcess(aExecutable, aArgs);
+    return LaunchWithIProcess(aExecutable, aArgc,
+                              reinterpret_cast<const char16_t**>(aArgv));
   }
 
   return NS_OK;
@@ -92,6 +94,35 @@ nsMIMEInfoWin::LaunchWithFile(nsIFile* aFile) {
                "nsMIMEInfoBase should have mClass == eMIMEInfo");
 
   if (mPreferredAction == useSystemDefault) {
+    if (mDefaultApplication &&
+        StaticPrefs::browser_pdf_launchDefaultEdgeAsApp()) {
+      // Since Edgium is the default PDF handler, if we're using the OS default
+      // and it's Edgium prefer it's app mode so it operates as a PDF viewer
+      // (without browser toolbars). Bug 1632277.
+      bool isPdf;
+      rv = IsPdf(&isPdf);
+      if (NS_SUCCEEDED(rv) && isPdf) {
+        nsAutoCString defaultAppExecutable;
+        rv = mDefaultApplication->GetNativeLeafName(defaultAppExecutable);
+        if (NS_SUCCEEDED(rv) &&
+            defaultAppExecutable.LowerCaseEqualsLiteral("msedge.exe")) {
+          nsAutoString path;
+          rv = aFile->GetPath(path);
+          if (NS_SUCCEEDED(rv)) {
+            // If the --app flag doesn't work we'll want to fallback to a
+            // regular path. Send two args so we call `msedge.exe --app={path}
+            // {path}`.
+            nsAutoString appArg;
+            appArg.AppendLiteral("--app=");
+            appArg.Append(path);
+            const wchar_t* argv[] = {appArg.get(), path.get()};
+
+            return ShellExecuteWithIFile(mDefaultApplication,
+                                         mozilla::ArrayLength(argv), argv);
+          }
+        }
+      }
+    }
     return LaunchDefaultWithFile(aFile);
   }
 
@@ -172,7 +203,8 @@ nsMIMEInfoWin::LaunchWithFile(nsIFile* aFile) {
     }
     nsAutoString path;
     aFile->GetPath(path);
-    return ShellExecuteWithIFile(executable, path);
+    const wchar_t* argv[] = {path.get()};
+    return ShellExecuteWithIFile(executable, mozilla::ArrayLength(argv), argv);
   }
 
   return NS_ERROR_INVALID_ARG;
@@ -281,13 +313,38 @@ nsresult nsMIMEInfoWin::LoadUriInternal(nsIURI* aURL) {
     _variant_t workingDir;
     _variant_t showCmd(SW_SHOWNORMAL);
 
-    // Ask Explorer to ShellExecute on our behalf, as some URL handlers do not
-    // start correctly when inheriting our process's process migitations.
+    // To open a uri, we first try ShellExecuteByExplorer, which starts a new
+    // process as a child process of explorer.exe, because applications may not
+    // support the mitigation policies inherited from our process.  If it fails,
+    // we fall back to ShellExecuteExW.
+    //
+    // For Thunderbird, however, there is a known issue that
+    // ShellExecuteByExplorer succeeds but explorer.exe shows an error popup
+    // if a uri to open includes credentials.  This does not happen in Firefox
+    // because Firefox does not have to launch a process to open a uri.
+    //
+    // Since Thunderbird does not use mitigation policies which could cause
+    // compatibility issues, we get no benefit from using
+    // ShellExecuteByExplorer.  Thus we skip it and go straight to
+    // ShellExecuteExW for Thunderbird.
+#ifndef MOZ_THUNDERBIRD
     mozilla::LauncherVoidResult shellExecuteOk =
-        mozilla::ShellExecuteByExplorer(validatedUri.unwrap(), args, verb,
+        mozilla::ShellExecuteByExplorer(validatedUri.inspect(), args, verb,
                                         workingDir, showCmd);
-    if (shellExecuteOk.isErr()) {
-      return NS_ERROR_FAILURE;
+    if (shellExecuteOk.isOk()) {
+      return NS_OK;
+    }
+#endif  // MOZ_THUNDERBIRD
+
+    SHELLEXECUTEINFOW sinfo = {sizeof(sinfo)};
+    sinfo.fMask = SEE_MASK_NOASYNC;
+    sinfo.lpVerb = V_BSTR(&verb);
+    sinfo.nShow = showCmd;
+    sinfo.lpFile = validatedUri.inspect();
+
+    BOOL result = ShellExecuteExW(&sinfo);
+    if (!result || reinterpret_cast<LONG_PTR>(sinfo.hInstApp) < 32) {
+      rv = NS_ERROR_FAILURE;
     }
   }
 
@@ -838,3 +895,20 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
 
   return NS_OK;
 }
+
+NS_IMETHODIMP
+nsMIMEInfoWin::IsCurrentAppOSDefault(bool* _retval) {
+  *_retval = false;
+  if (mDefaultApplication) {
+    // Determine if the default executable is our executable.
+    nsCOMPtr<nsIFile> ourBinary;
+    XRE_GetBinaryPath(getter_AddRefs(ourBinary));
+    bool isSame = false;
+    nsresult rv = mDefaultApplication->Equals(ourBinary, &isSame);
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
+    *_retval = isSame;
+  }
+  return NS_OK;
+}
