@@ -7,9 +7,9 @@
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/DebugOnly.h"
 #include "mozilla/UniquePtrExtensions.h"
+#include "mozilla/Utf8.h"
 
 #include "nsCOMPtr.h"
-#include "nsAutoPtr.h"
 #include "nsMemory.h"
 #include "GeckoProfiler.h"
 
@@ -19,7 +19,6 @@
 #include "nsNativeCharsetUtils.h"
 
 #include "nsSimpleEnumerator.h"
-#include "nsIComponentManager.h"
 #include "prio.h"
 #include "private/pprio.h"  // To get PR_ImportFile
 #include "nsHashKeys.h"
@@ -51,8 +50,8 @@
 #include "nsXPCOMCIDInternal.h"
 #include "nsThreadUtils.h"
 #include "nsXULAppAPI.h"
-
 #include "nsIWindowMediator.h"
+
 #include "mozIDOMWindow.h"
 #include "nsPIDOMWindow.h"
 #include "nsIWidget.h"
@@ -60,6 +59,8 @@
 #include "mozilla/WidgetUtils.h"
 
 using namespace mozilla;
+using mozilla::FilePreferences::kDevicePathSpecifier;
+using mozilla::FilePreferences::kPathSeparator;
 
 #define CHECK_mWorkingPath()                                     \
   do {                                                           \
@@ -79,6 +80,28 @@ using namespace mozilla;
 #  define DRIVE_REMOTE 4
 #endif
 
+namespace {
+
+nsresult NewLocalFile(const nsAString& aPath, bool aFollowLinks,
+                      bool aUseDOSDevicePathSyntax, nsIFile** aResult) {
+  RefPtr<nsLocalFile> file = new nsLocalFile();
+
+  file->SetFollowLinks(aFollowLinks);
+  file->SetUseDOSDevicePathSyntax(aUseDOSDevicePathSyntax);
+
+  if (!aPath.IsEmpty()) {
+    nsresult rv = file->InitWithPath(aPath);
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
+  }
+
+  file.forget(aResult);
+  return NS_OK;
+}
+
+}  // anonymous namespace
+
 static HWND GetMostRecentNavigatorHWND() {
   nsresult rv;
   nsCOMPtr<nsIWindowMediator> winMediator(
@@ -103,127 +126,69 @@ static HWND GetMostRecentNavigatorHWND() {
   return reinterpret_cast<HWND>(widget->GetNativeData(NS_NATIVE_WINDOW));
 }
 
-/**
- * A runnable to dispatch back to the main thread when
- * AsyncRevealOperation completes.
- */
-class AsyncLocalFileWinDone : public Runnable {
- public:
-  AsyncLocalFileWinDone()
-      : Runnable("AsyncLocalFileWinDone"),
-        mWorkerThread(do_GetCurrentThread()) {
-    // Objects of this type must only be created on worker threads
-    MOZ_ASSERT(!NS_IsMainThread());
+nsresult nsLocalFile::RevealFile(const nsString& aResolvedPath) {
+  MOZ_ASSERT(!NS_IsMainThread(), "Don't run on the main thread");
+
+  DWORD attributes = GetFileAttributesW(aResolvedPath.get());
+  if (INVALID_FILE_ATTRIBUTES == attributes) {
+    return NS_ERROR_FILE_INVALID_PATH;
   }
 
-  NS_IMETHOD Run() override {
-    // This event shuts down the worker thread and so must be main thread.
-    MOZ_ASSERT(NS_IsMainThread());
-
-    // If we don't destroy the thread when we're done with it, it will hang
-    // around forever... and that is bad!
-    mWorkerThread->Shutdown();
-    return NS_OK;
-  }
-
- private:
-  nsCOMPtr<nsIThread> mWorkerThread;
-};
-
-/**
- * A runnable to dispatch from the main thread when an async operation should
- * be performed.
- */
-class AsyncRevealOperation : public Runnable {
- public:
-  explicit AsyncRevealOperation(const nsAString& aResolvedPath)
-      : Runnable("AsyncRevealOperation"), mResolvedPath(aResolvedPath) {}
-
-  NS_IMETHOD Run() override {
-    MOZ_ASSERT(!NS_IsMainThread(),
-               "AsyncRevealOperation should not be run on the main thread!");
-
-    bool doCoUninitialize = SUCCEEDED(CoInitializeEx(
-        nullptr, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE));
-    Reveal();
-    if (doCoUninitialize) {
-      CoUninitialize();
+  HRESULT hr;
+  if (attributes & FILE_ATTRIBUTE_DIRECTORY) {
+    // We have a directory so we should open the directory itself.
+    LPITEMIDLIST dir = ILCreateFromPathW(aResolvedPath.get());
+    if (!dir) {
+      return NS_ERROR_FAILURE;
     }
 
-    // Send the result back to the main thread so that this thread can be
-    // cleanly shut down
-    nsCOMPtr<nsIRunnable> resultrunnable = new AsyncLocalFileWinDone();
-    NS_DispatchToMainThread(resultrunnable);
-    return NS_OK;
-  }
+    LPCITEMIDLIST selection[] = {dir};
+    UINT count = ArrayLength(selection);
 
- private:
-  // Reveals the path in explorer.
-  nsresult Reveal() {
-    DWORD attributes = GetFileAttributesW(mResolvedPath.get());
-    if (INVALID_FILE_ATTRIBUTES == attributes) {
+    // Perform the open of the directory.
+    hr = SHOpenFolderAndSelectItems(dir, count, selection, 0);
+    CoTaskMemFree(dir);
+  } else {
+    int32_t len = aResolvedPath.Length();
+    // We don't currently handle UNC long paths of the form \\?\ anywhere so
+    // this should be fine.
+    if (len > MAX_PATH) {
       return NS_ERROR_FILE_INVALID_PATH;
     }
+    WCHAR parentDirectoryPath[MAX_PATH + 1] = {0};
+    wcsncpy(parentDirectoryPath, aResolvedPath.get(), MAX_PATH);
+    PathRemoveFileSpecW(parentDirectoryPath);
 
-    HRESULT hr;
-    if (attributes & FILE_ATTRIBUTE_DIRECTORY) {
-      // We have a directory so we should open the directory itself.
-      LPITEMIDLIST dir = ILCreateFromPathW(mResolvedPath.get());
-      if (!dir) {
-        return NS_ERROR_FAILURE;
-      }
-
-      LPCITEMIDLIST selection[] = {dir};
-      UINT count = ArrayLength(selection);
-
-      // Perform the open of the directory.
-      hr = SHOpenFolderAndSelectItems(dir, count, selection, 0);
-      CoTaskMemFree(dir);
-    } else {
-      int32_t len = mResolvedPath.Length();
-      // We don't currently handle UNC long paths of the form \\?\ anywhere so
-      // this should be fine.
-      if (len > MAX_PATH) {
-        return NS_ERROR_FILE_INVALID_PATH;
-      }
-      WCHAR parentDirectoryPath[MAX_PATH + 1] = {0};
-      wcsncpy(parentDirectoryPath, mResolvedPath.get(), MAX_PATH);
-      PathRemoveFileSpecW(parentDirectoryPath);
-
-      // We have a file so we should open the parent directory.
-      LPITEMIDLIST dir = ILCreateFromPathW(parentDirectoryPath);
-      if (!dir) {
-        return NS_ERROR_FAILURE;
-      }
-
-      // Set the item in the directory to select to the file we want to reveal.
-      LPITEMIDLIST item = ILCreateFromPathW(mResolvedPath.get());
-      if (!item) {
-        CoTaskMemFree(dir);
-        return NS_ERROR_FAILURE;
-      }
-
-      LPCITEMIDLIST selection[] = {item};
-      UINT count = ArrayLength(selection);
-
-      // Perform the selection of the file.
-      hr = SHOpenFolderAndSelectItems(dir, count, selection, 0);
-
-      CoTaskMemFree(dir);
-      CoTaskMemFree(item);
+    // We have a file so we should open the parent directory.
+    LPITEMIDLIST dir = ILCreateFromPathW(parentDirectoryPath);
+    if (!dir) {
+      return NS_ERROR_FAILURE;
     }
 
-    return SUCCEEDED(hr) ? NS_OK : NS_ERROR_FAILURE;
+    // Set the item in the directory to select to the file we want to reveal.
+    LPITEMIDLIST item = ILCreateFromPathW(aResolvedPath.get());
+    if (!item) {
+      CoTaskMemFree(dir);
+      return NS_ERROR_FAILURE;
+    }
+
+    LPCITEMIDLIST selection[] = {item};
+    UINT count = ArrayLength(selection);
+
+    // Perform the selection of the file.
+    hr = SHOpenFolderAndSelectItems(dir, count, selection, 0);
+
+    CoTaskMemFree(dir);
+    CoTaskMemFree(item);
   }
 
-  // Stores the path to perform the operation on
-  nsString mResolvedPath;
-};
+  return SUCCEEDED(hr) ? NS_OK : NS_ERROR_FAILURE;
+}
 
 class nsDriveEnumerator : public nsSimpleEnumerator,
                           public nsIDirectoryEnumerator {
  public:
-  nsDriveEnumerator();
+  explicit nsDriveEnumerator(bool aUseDOSDevicePathSyntax);
   NS_DECL_ISUPPORTS_INHERITED
   NS_DECL_NSISIMPLEENUMERATOR
   NS_FORWARD_NSISIMPLEENUMERATORBASE(nsSimpleEnumerator::)
@@ -259,6 +224,7 @@ class nsDriveEnumerator : public nsSimpleEnumerator,
   nsString mDrives;
   nsAString::const_iterator mStartOfCurrentDrive;
   nsAString::const_iterator mEndOfDrivesString;
+  const bool mUseDOSDevicePathSyntax;
 };
 
 //-----------------------------------------------------------------------------
@@ -504,12 +470,21 @@ static void FileTimeToPRTime(const FILETIME* aFiletime, PRTime* aPrtm) {
 // copied from nsprpub/pr/src/{io/prfile.c | md/windows/w95io.c} with some
 // changes : PR_GetFileInfo64, _PR_MD_GETFILEINFO64
 static nsresult GetFileInfo(const nsString& aName, PRFileInfo64* aInfo) {
-  WIN32_FILE_ATTRIBUTE_DATA fileData;
-
-  if (aName.IsEmpty() || aName.FindCharInSet(u"?*") != kNotFound) {
+  if (aName.IsEmpty()) {
     return NS_ERROR_INVALID_ARG;
   }
 
+  // Checking u"?*" for the file path excluding the kDevicePathSpecifier.
+  // ToDo: Check if checking "?" for the file path is still needed.
+  const int32_t offset = StringBeginsWith(aName, kDevicePathSpecifier)
+                             ? kDevicePathSpecifier.Length()
+                             : 0;
+
+  if (aName.FindCharInSet(u"?*", offset) != kNotFound) {
+    return NS_ERROR_INVALID_ARG;
+  }
+
+  WIN32_FILE_ATTRIBUTE_DATA fileData;
   if (!::GetFileAttributesExW(aName.get(), GetFileExInfoStandard, &fileData)) {
     return ConvertWinError(GetLastError());
   }
@@ -547,9 +522,6 @@ static nsresult OpenDir(const nsString& aName, nsDir** aDir) {
   }
 
   *aDir = nullptr;
-  if (aName.Length() + 3 >= MAX_PATH) {
-    return NS_ERROR_FILE_NAME_TOO_LONG;
-  }
 
   nsDir* d = new nsDir();
   nsAutoString filename(aName);
@@ -763,9 +735,13 @@ NS_IMPL_ISUPPORTS_INHERITED(nsDirEnumerator, nsSimpleEnumerator,
 //-----------------------------------------------------------------------------
 
 nsLocalFile::nsLocalFile()
-    : mDirty(true), mResolveDirty(true), mFollowSymlinks(false) {}
+    : mDirty(true),
+      mResolveDirty(true),
+      mFollowSymlinks(false),
+      mUseDOSDevicePathSyntax(false) {}
 
-nsLocalFile::nsLocalFile(const nsAString& aFilePath) : mFollowSymlinks(false) {
+nsLocalFile::nsLocalFile(const nsAString& aFilePath)
+    : mFollowSymlinks(false), mUseDOSDevicePathSyntax(false) {
   InitWithPath(aFilePath);
 }
 
@@ -802,6 +778,7 @@ nsLocalFile::nsLocalFile(const nsLocalFile& aOther)
     : mDirty(true),
       mResolveDirty(true),
       mFollowSymlinks(aOther.mFollowSymlinks),
+      mUseDOSDevicePathSyntax(aOther.mUseDOSDevicePathSyntax),
       mWorkingPath(aOther.mWorkingPath) {}
 
 // Resolve the shortcut file from mWorkingPath and write the path
@@ -996,6 +973,15 @@ nsLocalFile::InitWithPath(const nsAString& aFilePath) {
     mWorkingPath.Truncate(mWorkingPath.Length() - 1);
   }
 
+  // Bug 1626514: make sure that we don't end up with multiple prefixes.
+
+  // Prepend the "\\?\" prefix if the useDOSDevicePathSyntax is set and the path
+  // starts with a disk designator and backslash.
+  if (mUseDOSDevicePathSyntax &&
+      FilePreferences::StartsWithDiskDesignatorAndBackslash(mWorkingPath)) {
+    mWorkingPath = kDevicePathSpecifier + mWorkingPath;
+  }
+
   return NS_OK;
 }
 
@@ -1527,6 +1513,9 @@ nsLocalFile::SetLeafName(const nsAString& aLeafName) {
 
 NS_IMETHODIMP
 nsLocalFile::GetPath(nsAString& aResult) {
+  MOZ_ASSERT_IF(
+      mUseDOSDevicePathSyntax,
+      !FilePreferences::StartsWithDiskDesignatorAndBackslash(mWorkingPath));
   aResult = mWorkingPath;
   return NS_OK;
 }
@@ -1677,6 +1666,24 @@ nsresult nsLocalFile::CopySingleFile(nsIFile* aSourceFile, nsIFile* aDestParent,
     return rv;
   }
 
+#ifdef DEBUG
+  nsCOMPtr<nsILocalFileWin> srcWinFile = do_QueryInterface(aSourceFile);
+  MOZ_ASSERT(srcWinFile);
+
+  bool srcUseDOSDevicePathSyntax;
+  srcWinFile->GetUseDOSDevicePathSyntax(&srcUseDOSDevicePathSyntax);
+
+  nsCOMPtr<nsILocalFileWin> destWinFile = do_QueryInterface(aDestParent);
+  MOZ_ASSERT(destWinFile);
+
+  bool destUseDOSDevicePathSyntax;
+  destWinFile->GetUseDOSDevicePathSyntax(&destUseDOSDevicePathSyntax);
+
+  MOZ_ASSERT(srcUseDOSDevicePathSyntax == destUseDOSDevicePathSyntax,
+             "Copy or Move files with different values for "
+             "useDOSDevicePathSyntax would fail");
+#endif
+
   if (FilePreferences::IsBlockedUNCPath(destPath)) {
     return NS_ERROR_FILE_ACCESS_DENIED;
   }
@@ -2606,8 +2613,8 @@ nsLocalFile::GetParent(nsIFile** aParent) {
 
   nsCOMPtr<nsIFile> localFile;
   nsresult rv =
-      NS_NewLocalFile(parentPath, mFollowSymlinks, getter_AddRefs(localFile));
-
+      NewLocalFile(parentPath, mFollowSymlinks, mUseDOSDevicePathSyntax,
+                   getter_AddRefs(localFile));
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -2706,8 +2713,8 @@ nsLocalFile::IsReadable(bool* aResult) {
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsLocalFile::IsExecutable(bool* aResult) {
+nsresult nsLocalFile::LookupExtensionIn(const char* const* aExtensionsArray,
+                                        size_t aArrayLength, bool* aResult) {
   // Check we are correctly initialized.
   CHECK_mWorkingPath();
 
@@ -2759,8 +2766,8 @@ nsLocalFile::IsExecutable(bool* aResult) {
     }
 
     nsDependentSubstring ext = Substring(path, dotIdx);
-    for (size_t i = 0; i < ArrayLength(sExecutableExts); ++i) {
-      if (ext.EqualsASCII(sExecutableExts[i])) {
+    for (size_t i = 0; i < aArrayLength; ++i) {
+      if (ext.EqualsASCII(aExtensionsArray[i])) {
         // Found a match.  Set result and quit.
         *aResult = true;
         break;
@@ -2771,6 +2778,12 @@ nsLocalFile::IsExecutable(bool* aResult) {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsLocalFile::IsExecutable(bool* aResult) {
+  return LookupExtensionIn(sExecutableExts, ArrayLength(sExecutableExts),
+                           aResult);
+}
+
 NS_IMETHODIMP
 nsLocalFile::IsDirectory(bool* aResult) {
   return HasFileAttribute(FILE_ATTRIBUTE_DIRECTORY, aResult);
@@ -2863,8 +2876,32 @@ nsLocalFile::Equals(nsIFile* aInFile, bool* aResult) {
   nsAutoString inFilePath;
   lf->GetCanonicalPath(inFilePath);
 
+  bool inUseDOSDevicePathSyntax;
+  lf->GetUseDOSDevicePathSyntax(&inUseDOSDevicePathSyntax);
+
+  // Remove the prefix for both inFilePath and mShortWorkingPath if the
+  // useDOSDevicePathSyntax from them are not the same.
+  // This is added because of Omnijar. It compare files from different moduals
+  // with itself
+  nsAutoString shortWorkingPath;
+  if (inUseDOSDevicePathSyntax == mUseDOSDevicePathSyntax) {
+    shortWorkingPath = mShortWorkingPath;
+  } else if (inUseDOSDevicePathSyntax &&
+             StringBeginsWith(inFilePath, kDevicePathSpecifier)) {
+    MOZ_ASSERT(!StringBeginsWith(mShortWorkingPath, kDevicePathSpecifier));
+
+    shortWorkingPath = mShortWorkingPath;
+    inFilePath = Substring(inFilePath, kDevicePathSpecifier.Length());
+  } else if (mUseDOSDevicePathSyntax &&
+             StringBeginsWith(mShortWorkingPath, kDevicePathSpecifier)) {
+    MOZ_ASSERT(!StringBeginsWith(inFilePath, kDevicePathSpecifier));
+
+    shortWorkingPath =
+        Substring(mShortWorkingPath, kDevicePathSpecifier.Length());
+  }
+
   // Ok : Win9x
-  *aResult = _wcsicmp(mShortWorkingPath.get(), inFilePath.get()) == 0;
+  *aResult = _wcsicmp(shortWorkingPath.get(), inFilePath.get()) == 0;
 
   return NS_OK;
 }
@@ -2904,6 +2941,10 @@ nsLocalFile::GetTarget(nsAString& aResult) {
   aResult.Truncate();
   Resolve();
 
+  MOZ_ASSERT_IF(
+      mUseDOSDevicePathSyntax,
+      !FilePreferences::StartsWithDiskDesignatorAndBackslash(mResolvedPath));
+
   aResult = mResolvedPath;
   return NS_OK;
 }
@@ -2926,7 +2967,8 @@ nsLocalFile::GetDirectoryEntriesImpl(nsIDirectoryEnumerator** aEntries) {
 
   *aEntries = nullptr;
   if (mWorkingPath.EqualsLiteral("\\\\.")) {
-    RefPtr<nsDriveEnumerator> drives = new nsDriveEnumerator;
+    RefPtr<nsDriveEnumerator> drives =
+        new nsDriveEnumerator(mUseDOSDevicePathSyntax);
     rv = drives->Init();
     if (NS_FAILED(rv)) {
       return rv;
@@ -2954,7 +2996,7 @@ nsLocalFile::GetPersistentDescriptor(nsACString& aPersistentDescriptor) {
 
 NS_IMETHODIMP
 nsLocalFile::SetPersistentDescriptor(const nsACString& aPersistentDescriptor) {
-  if (IsUTF8(aPersistentDescriptor)) {
+  if (IsUtf8(aPersistentDescriptor)) {
     return InitWithPath(NS_ConvertUTF8toUTF16(aPersistentDescriptor));
   } else {
     return InitWithNativePath(aPersistentDescriptor);
@@ -3002,6 +3044,38 @@ nsLocalFile::SetFileAttributesWin(uint32_t aAttribs) {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsLocalFile::GetUseDOSDevicePathSyntax(bool* aUseDOSDevicePathSyntax) {
+  MOZ_ASSERT(aUseDOSDevicePathSyntax);
+
+  *aUseDOSDevicePathSyntax = mUseDOSDevicePathSyntax;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsLocalFile::SetUseDOSDevicePathSyntax(bool aUseDOSDevicePathSyntax) {
+  if (mUseDOSDevicePathSyntax == aUseDOSDevicePathSyntax) {
+    return NS_OK;
+  }
+
+  if (mUseDOSDevicePathSyntax) {
+    if (StringBeginsWith(mWorkingPath, kDevicePathSpecifier)) {
+      MakeDirty();
+      // Remove the prefix
+      mWorkingPath = Substring(mWorkingPath, kDevicePathSpecifier.Length());
+    }
+  } else {
+    if (FilePreferences::StartsWithDiskDesignatorAndBackslash(mWorkingPath)) {
+      MakeDirty();
+      // Prepend the prefix
+      mWorkingPath = kDevicePathSpecifier + mWorkingPath;
+    }
+  }
+
+  mUseDOSDevicePathSyntax = aUseDOSDevicePathSyntax;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsLocalFile::Reveal() {
   // This API should be main thread only
@@ -3013,20 +3087,20 @@ nsLocalFile::Reveal() {
     return rv;
   }
 
-  // To create a new thread, get the thread manager
-  nsCOMPtr<nsIThreadManager> tm = do_GetService(NS_THREADMANAGER_CONTRACTID);
-  nsCOMPtr<nsIThread> mythread;
-  rv = tm->NewThread(0, 0, getter_AddRefs(mythread));
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
+  nsCOMPtr<nsIRunnable> task =
+      NS_NewRunnableFunction("nsLocalFile::Reveal", [path = mResolvedPath]() {
+        MOZ_ASSERT(!NS_IsMainThread(), "Don't run on the main thread");
 
-  nsCOMPtr<nsIRunnable> runnable = new AsyncRevealOperation(mResolvedPath);
+        bool doCoUninitialize = SUCCEEDED(CoInitializeEx(
+            nullptr, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE));
+        RevealFile(path);
+        if (doCoUninitialize) {
+          CoUninitialize();
+        }
+      });
 
-  // After the dispatch, the result runnable will shut down the worker
-  // thread, so we can let it go.
-  mythread->Dispatch(runnable, NS_DISPATCH_NORMAL);
-  return NS_OK;
+  return NS_DispatchBackgroundTask(task,
+                                   nsIEventTarget::DISPATCH_EVENT_MAY_BLOCK);
 }
 
 NS_IMETHODIMP
@@ -3061,29 +3135,71 @@ nsLocalFile::Launch() {
     NS_WARNING("Could not set working directory for launched file.");
   }
 
-  // Ask Explorer to ShellExecute on our behalf, as some applications such as
-  // Skype for Business do not start correctly when inheriting our process's
-  // migitation policies.
-  // It does not work in a special environment such as Citrix.  In such a case
-  // we fall back to launching an application as a child process.  We need to
-  // find a way to handle the combination of these interop issues.
-  mozilla::LauncherVoidResult shellExecuteOk = mozilla::ShellExecuteByExplorer(
-      execPath, args, verbDefault, workingDirectoryPtr, showCmd);
-  if (shellExecuteOk.isErr()) {
-    SHELLEXECUTEINFOW seinfo = {sizeof(SHELLEXECUTEINFOW)};
-    seinfo.fMask = SEE_MASK_ASYNCOK;
-    seinfo.hwnd = GetMostRecentNavigatorHWND();
-    seinfo.lpVerb = nullptr;
-    seinfo.lpFile = mResolvedPath.get();
-    seinfo.lpParameters = nullptr;
-    seinfo.lpDirectory = workingDirectoryPtr;
-    seinfo.nShow = SW_SHOWNORMAL;
+  // We have two methods to launch a file: ShellExecuteExW and
+  // ShellExecuteByExplorer.  ShellExecuteExW starts a new process as a child
+  // of the current process, while ShellExecuteByExplorer starts a new process
+  // as a child of explorer.exe.
+  //
+  // We prefer launching a process via ShellExecuteByExplorer because
+  // applications may not support the mitigation policies inherited from our
+  // process.  For example, Skype for Business does not start correctly with
+  // the PreferSystem32Images policy which is one of the policies we use.
+  //
+  // If ShellExecuteByExplorer fails for some reason e.g. a system without
+  // running explorer.exe or VDI environment like Citrix, we fall back to
+  // ShellExecuteExW which still works in those special environments.
+  //
+  // There is an exception where we go straight to ShellExecuteExW without
+  // trying ShellExecuteByExplorer.  When the extension of a downloaded file is
+  // "exe", we prefer security rather than compatibility.
+  //
+  // When a user launches a downloaded executable, the directory containing
+  // the downloaded file may contain a malicious DLL with a common name, which
+  // may have been downloaded before.  If the downloaded executable is launched
+  // without the PreferSystem32Images policy, the process can be tricked into
+  // loading the malicious DLL in the same directory if its name is in the
+  // executable's dependent modules.  Therefore, we always launch ".exe"
+  // executables via ShellExecuteExW so they inherit our process's mitigation
+  // policies including PreferSystem32Images.
+  //
+  // If the extension is not "exe", then we assume that we are launching an
+  // installed application, and therefore the security risk described above
+  // is lessened, as a malicious DLL is less likely to be installed in the
+  // application's directory.  In that case, we attempt to preserve
+  // compatibility and try ShellExecuteByExplorer first.
 
-    if (!ShellExecuteExW(&seinfo)) {
-      return NS_ERROR_FILE_EXECUTION_FAILED;
+  static const char* const onlyExeExt[] = {".exe"};
+  bool isExecutable;
+  rv = LookupExtensionIn(onlyExeExt, ArrayLength(onlyExeExt), &isExecutable);
+  if (NS_FAILED(rv)) {
+    isExecutable = false;
+  }
+
+  // If the file is an executable, go straight to ShellExecuteExW.
+  // Otherwise try ShellExecuteByExplorer first, and if it fails,
+  // run ShellExecuteExW.
+  if (!isExecutable) {
+    mozilla::LauncherVoidResult shellExecuteOk =
+        mozilla::ShellExecuteByExplorer(execPath, args, verbDefault,
+                                        workingDirectoryPtr, showCmd);
+    if (shellExecuteOk.isOk()) {
+      return NS_OK;
     }
   }
 
+  SHELLEXECUTEINFOW seinfo = {sizeof(SHELLEXECUTEINFOW)};
+  seinfo.fMask = SEE_MASK_ASYNCOK;
+  seinfo.hwnd = GetMostRecentNavigatorHWND();
+  seinfo.lpVerb = nullptr;
+  seinfo.lpFile = mResolvedPath.get();
+  seinfo.lpParameters = nullptr;
+  seinfo.lpDirectory = workingDirectoryPtr;
+  seinfo.nShow = SW_SHOWNORMAL;
+
+  if (!ShellExecuteExW(&seinfo)) {
+    return NS_ERROR_FILE_EXECUTION_FAILED;
+  }
+
   return NS_OK;
 }
 
@@ -3279,7 +3395,8 @@ void nsLocalFile::EnsureShortPath() {
 NS_IMPL_ISUPPORTS_INHERITED(nsDriveEnumerator, nsSimpleEnumerator,
                             nsIDirectoryEnumerator)
 
-nsDriveEnumerator::nsDriveEnumerator() {}
+nsDriveEnumerator::nsDriveEnumerator(bool aUseDOSDevicePathSyntax)
+    : mUseDOSDevicePathSyntax(aUseDOSDevicePathSyntax) {}
 
 nsDriveEnumerator::~nsDriveEnumerator() {}
 
@@ -3323,7 +3440,7 @@ nsDriveEnumerator::GetNext(nsISupports** aNext) {
   mStartOfCurrentDrive = ++driveEnd;
 
   nsIFile* file;
-  nsresult rv = NS_NewLocalFile(drive, false, &file);
+  nsresult rv = NewLocalFile(drive, false, mUseDOSDevicePathSyntax, &file);
 
   *aNext = file;
   return rv;
